"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewDecoder = void 0;
var tslib_1 = require("tslib");
var CrdtDecoder_1 = require("../../../json-crdt-patch/util/binary/CrdtDecoder");
var ViewDecoder = (function (_super) {
    (0, tslib_1.__extends)(ViewDecoder, _super);
    function ViewDecoder() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.time = -1;
        return _this;
    }
    ViewDecoder.prototype.decode = function (data) {
        this.time = -1;
        this.reset(data);
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), isServerTime = _a[0], x = _a[1];
        if (isServerTime) {
            this.time = x;
        }
        else {
            this.decodeClockTable(x);
        }
        this.literals = this.val();
        if (!(this.literals instanceof Array))
            throw new Error('INVALID_LITERALS');
        return this.decodeRoot();
    };
    ViewDecoder.prototype.decodeClockTable = function (length) {
        this.uint53vuint39();
        for (var i = 1; i < length; i++)
            this.uint53vuint39();
    };
    ViewDecoder.prototype.ts = function () {
        if (this.time < 0) {
            this.id();
        }
        else {
            var _a = (0, tslib_1.__read)(this.b1vuint56(), 1), isServerTime = _a[0];
            if (isServerTime) {
            }
            else {
                this.vuint57();
            }
        }
    };
    ViewDecoder.prototype.decodeRoot = function () {
        this.ts();
        return this.decodeNode();
    };
    ViewDecoder.prototype.decodeNode = function () {
        var byte = this.u8();
        if (byte < 128)
            return this.createConst(byte);
        else if (byte <= 143)
            return this.decodeObj(byte & 15);
        else if (byte <= 159)
            return this.decodeArr(byte & 15);
        else if (byte <= 191)
            return this.decodeStr(byte & 31);
        else if (byte >= 224)
            return this.createConst(byte - 0x100);
        else {
            switch (byte) {
                case 0xc0:
                    return null;
                case 0xc1:
                    return undefined;
                case 0xc2:
                    return false;
                case 0xc3:
                    return true;
                case 0xc4:
                    return this.decodeBin(this.u8());
                case 0xc5:
                    return this.decodeBin(this.u16());
                case 0xc6:
                    return this.decodeBin(this.u32());
                case 0xca:
                    return this.f32();
                case 0xcb:
                    return this.f64();
                case 0xcc:
                    return this.u8();
                case 0xcd:
                    return this.u16();
                case 0xce:
                    return this.u32();
                case 0xcf:
                    return this.u32() * 4294967296 + this.u32();
                case 0xd0:
                    return this.i8();
                case 0xd1:
                    return this.i16();
                case 0xd2:
                    return this.i32();
                case 0xd3:
                    return this.i32() * 4294967296 + this.i32();
                case 0xd4:
                    return this.val();
                case 0xd5: {
                    this.ts();
                    this.ts();
                    return this.val();
                }
                case 0xd6: {
                    this.ts();
                    this.ts();
                    var literalIndex = this.vuint57();
                    return this.literals[literalIndex];
                }
                case 0xde:
                    return this.decodeObj(this.u16());
                case 0xdf:
                    return this.decodeObj(this.u32());
                case 0xdc:
                    return this.decodeArr(this.u16());
                case 0xdd:
                    return this.decodeArr(this.u32());
                case 0xd9:
                    return this.decodeStr(this.u8());
                case 0xda:
                    return this.decodeStr(this.u16());
                case 0xdb:
                    return this.decodeStr(this.u32());
            }
        }
        return null;
    };
    ViewDecoder.prototype.createConst = function (value) {
        return value;
    };
    ViewDecoder.prototype.decodeObj = function (length) {
        this.ts();
        var obj = {};
        for (var i = 0; i < length; i++) {
            this.ts();
            var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), isLiteralIndex = _a[0], x = _a[1];
            var key = void 0;
            if (isLiteralIndex) {
                key = this.literals[x];
            }
            else {
                key = this.str(x);
            }
            var value = this.decodeNode();
            if (value !== undefined)
                obj[key] = value;
        }
        return obj;
    };
    ViewDecoder.prototype.decodeArr = function (length) {
        this.ts();
        var arr = [];
        for (var i = 0; i < length; i++) {
            var values = this.decodeArrChunk();
            if (values && values.length)
                arr.push.apply(arr, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(values), false));
        }
        return arr;
    };
    ViewDecoder.prototype.decodeArrChunk = function () {
        var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length = _a[1];
        this.ts();
        if (deleted) {
            return undefined;
        }
        else {
            var values = [];
            for (var i = 0; i < length; i++)
                values.push(this.decodeNode());
            return values;
        }
    };
    ViewDecoder.prototype.decodeStr = function (length) {
        this.ts();
        var str = '';
        for (var i = 0; i < length; i++) {
            var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length_1 = _a[1];
            this.ts();
            if (deleted)
                continue;
            if (length_1 === 0)
                str += this.literals[this.vuint57()];
            str += this.str(length_1);
        }
        return str;
    };
    ViewDecoder.prototype.decodeBin = function (length) {
        this.ts();
        var buffers = [];
        var totalLength = 0;
        for (var i = 0; i < length; i++) {
            var _a = (0, tslib_1.__read)(this.b1vuint56(), 2), deleted = _a[0], length_2 = _a[1];
            this.ts();
            if (deleted)
                continue;
            buffers.push(this.bin(length_2));
            totalLength += length_2;
        }
        var res = new Uint8Array(totalLength);
        var offset = 0;
        for (var i = 0; i < buffers.length; i++) {
            var byteLength = buffers[i].byteLength;
            res.set(buffers[i], offset);
            offset += byteLength;
        }
        return res;
    };
    return ViewDecoder;
}(CrdtDecoder_1.CrdtDecoder));
exports.ViewDecoder = ViewDecoder;
